const async = require('async');
const dynamoWrapper = require('dynamodb-data-types').AttributeValue;
const dynamoDBCapacity =  5;
const pg = require('pg').Client;
const sql = require('sql-bricks-postgres');
const moment = require('moment');
exports.handler = function(event, context, callback) {

    const client = new pg({
        host: process.env.DB_HOST,//'fndy-search-result.cildgunnxgsj.us-west-2.rds.amazonaws.com',
        port: 5432,
        user: 'search_user',
        password: '0bdf3d47c38a3d61b',
        database: 'AdvancedSearch'
    });

    function query(str, callback){
        client.query(str, function(err, data){

            if (!err) {
                return callback(null, data.rows?data.rows:data);
            } else {
                console.log(err);
                console.log(str);
                return callback(err, data);
            }
        })

    }

    let clientConnected = false;

    function init(callbackInit) {
        connectRdsClient(err => callbackInit(err, event.Records))
    }

    function commonCycle(records, commonCycleCallback) {
        async.mapLimit(records, dynamoDBCapacity, recordAnalysis, function(err, result) {
            if (err) {
                return commonCycleCallback(err);
            }

            return commonCycleCallback(null);
        });
    }

    function connectRdsClient(cb){
        if (clientConnected) return cb(null);
        client.connect(function(err) {
            if (err) {
                console.error('connection error', err.stack);
                return cb('PG connect error');
            } else {
                clientConnected = true;
                cb(null)
            }
        });
    }

    let columnsByTypeId = {};

    function GetColumnsFromDatatable(typeId, cbGetColumns){

        if (columnsByTypeId[typeId]) return cbGetColumns(null, columnsByTypeId[typeId]);

        const request = sql.select("column_name, data_type").from("information_schema.columns").where(sql.and(sql.like("column_name", 'a.%'),{table_schema:"public",table_name:`_history_type_${typeId}`})).toString();

        query(request, function(err,data){
            if (err) return cbGetColumns(err);

            data.forEach(c => {
                c.data_type = c.data_type === 'bigint'?'Integer':c.data_type === 'numeric'?'Decimal':'String'
            });

            columnsByTypeId[typeId] = data;
            cbGetColumns(err, data);
        });

    }

    const mandatoryFields = {
        deviceId: 'String',
        accountId: 'String',
        typeId: 'String',
        epochTime: 'Integer',
    };

    function recordAnalysis(record, recordAnalysisCallback) {
        if (record.dynamodb.NewImage
            && (record.eventName === 'INSERT' || record.eventName === 'MODIFY')) {

            const wrappedNew = record.dynamodb.NewImage;
            const unwrappedNew = dynamoWrapper.unwrap(wrappedNew);

            GetColumnsFromDatatable(unwrappedNew.typeId, (err, columns) => {
                if (err) return recordAnalysisCallback(null);

                let insertFieldsExpression = '';
                let insertValuesExpression = '';
                let updateExpression = '';

                unwrappedNew.epochTime = moment().valueOf();

                for(let fieldName in mandatoryFields){

                    if(fieldName in unwrappedNew){

                        let correctValueRecord = getCorrectInsertRecord(fieldName, mandatoryFields[fieldName]);

                        insertFieldsExpression += `"${fieldName}", `;
                        insertValuesExpression += correctValueRecord;
                        updateExpression += `"${fieldName}" = ${correctValueRecord}`;
                    }
                }

                columns.forEach(column => {

                    if(column.column_name in unwrappedNew){

                        let correctValueRecord = getCorrectInsertRecord(column.column_name, column.data_type);

                        insertFieldsExpression += `"${column.column_name}", `;
                        insertValuesExpression += correctValueRecord;
                        updateExpression += `"${column.column_name}" = ${correctValueRecord}`;
                    }
                });

                let requestString = `INSERT INTO "_history_type_${unwrappedNew.typeId}" (${insertFieldsExpression.slice(0, -2)})
                                    VALUES (${insertValuesExpression.slice(0, -2)});`;

                query(requestString, (err, res) => {
                    return recordAnalysisCallback(null);
                });

            });

            function convertToCorrectFormat(value, dataType){

                switch (dataType) {
                    case "Integer":
                        let int_val = parseInt(value);
                        return isNaN(int_val) || int_val=== undefined? null: int_val;
                    case "Decimal":
                        let decimal_val = +value;
                        return isNaN(decimal_val) || decimal_val=== undefined? null: decimal_val;
                    default:
                        return value !== undefined && value !== null? (""+value).replace(/'/g,"''").replace(/\u0000/g,""): null;
                }
            }

            function getCorrectInsertRecord (fieldName, dataType) {
                const value = convertToCorrectFormat(unwrappedNew[fieldName],dataType);
                switch (dataType) {
                    case "Integer":
                    case "Decimal":
                        return `${value}, `;
                    case "Boolean":
                        if (value) return `true, `;
                        else return `false, `;
                    default:
                        return `${value !== undefined && value !== null? "'"+(""+value).replace(/'/g,"''").replace(/\u0000/g,"")+"'": null}, `;
                }
            }



        }else {
            return recordAnalysisCallback(null);
        }
    }

    async.waterfall([
        init,
        commonCycle
    ], function (err, result) {

        if (clientConnected) client.end();

        if (err) {
            console.error('error:', err);
            const response = {
                status: 400,
                error: err.message ? err.message : err
            };
            return context.succeed(JSON.stringify(response));
        }

        return context.succeed(result);

    });
};
/*
CREATE TABLE "_history_type_16dc7e50-e4fc-11e7-97cd-d17b4174b15e"
(
    "deviceId"             VARCHAR(50),
    "accountId"             VARCHAR(50),
    "epochTime"            BIGINT,
    "a.email_alert"      TEXT,
    "a.name"             TEXT,
    "a.reference_object" VARCHAR(50),
    "a.text_alert"       TEXT
);

CREATE TABLE "_history_type_d52284f0-e4f1-11e7-97cd-d17b4174b15e"
(
  "a.air_temp_alarm_count"            NUMERIC,
  "a.air_temperature"                 NUMERIC,
  "a.air_temperature_alarm"           NUMERIC,
  "a.alarm_status"                    VARCHAR(50),
  "a.barometric_pressure"             NUMERIC,
  "a.create_air_temperature_alarm"    NUMERIC,
  "a.datetime"                        BIGINT,
  "a.increment_air_temperature_count" NUMERIC,
  "a.lat"                             VARCHAR(50),
  "a.long"                            VARCHAR(50),
  "a.location"                        TEXT,
  "a.max_wind_speed"                  NUMERIC,
  "a.name"                            TEXT,
  "a.rainfall"                        TEXT,
  "a.relative_humidity"               NUMERIC,
  "a.temperature_celsius"             NUMERIC,
  "a.wind_direction"                  NUMERIC,
  "a.wind_speed"                      NUMERIC,
  "deviceId"                            VARCHAR(50),
  "epochTime"                           BIGINT,
  "accountId"                           VARCHAR(50),
  "a.average_temp"      NUMERIC,
  "a.barometric_pressure"   NUMERIC,
  "a.date"  TEXT,

)

*/